/*
 * Copyright (c) 2021 Nordic Semiconductor ASA
 *
 * SPDX-License-Identifier: Apache-2.0
 */


/* general includes =========================================================================================*/
#include <zephyr/sys/printk.h>

#include <zephyr/device.h>
#include <zephyr/drivers/gpio.h>
#include <zephyr/drivers/pinctrl.h>
#include <zephyr/drivers/i2c.h>

//Power Management
#include <pm/pm.h>
#include <pm/device.h>
#include <pm/policy.h>

//Sensor
#include <zephyr.h>
#include <device.h>
#include <drivers/sensor.h>
//#include <drivers/sensor/lsm6dso.h>


//Rotation 
#include "IMU/IMU.h"

// Pressure
#include "Pressure/Pressure.h"

/* defines ======================================================================================*/

#define DEVICE_NAME_PRESSURE 			meas_ms5607
#define DEVICE_NAME_ADC					"ADC"
#define GPIO_DEV_NAME1 					"GPIO_0"
#define REGULATOR_IMU 					"imu-pwr-ctrl"
#define REGULATOR_PRESSURE 				"pressure-pwr-ctrl"
#define GPIO_I2C_DEV_NAME 				"i2c1_gpio0"
#define DEVICE_NAME_I2C					"i2c0"
#define VDD_PIN_IMU 					4 //1
#define VDD_PIN_PRESSURE 				14 //0 

/* Pincontrol related stuff ===================================================================================*/

/* make sure devices and remap hook are initialized in the correct order */
BUILD_ASSERT((CONFIG_GPIO_INIT_PRIORITY < CONFIG_REMAP_INIT_PRIORITY) &&
	     (CONFIG_REMAP_INIT_PRIORITY < CONFIG_SERIAL_INIT_PRIORITY),
	     "Device driver priorities are not set correctly");

/* UART0 alternative configurations (default and sleep states) */
PINCTRL_DT_STATE_PINS_DEFINE(DT_PATH(zephyr_user), uart0_alt_default);
PINCTRL_DT_STATE_PINS_DEFINE(DT_PATH(zephyr_user), i2c0_alt_default);
PINCTRL_DT_STATE_PINS_DEFINE(DT_PATH(zephyr_user), i2c0_default);

#ifdef CONFIG_PM_DEVICE
PINCTRL_DT_STATE_PINS_DEFINE(DT_PATH(zephyr_user), uart0_alt_sleep);
PINCTRL_DT_STATE_PINS_DEFINE(DT_PATH(zephyr_user), i2c0_alt_sleep);
PINCTRL_DT_STATE_PINS_DEFINE(DT_PATH(zephyr_user), i2c0_sleep);
#endif

PINCTRL_DT_DEV_CONFIG_DECLARE(DT_NODELABEL(uart0));
PINCTRL_DT_DEV_CONFIG_DECLARE(DT_NODELABEL(i2c0));

struct pinctrl_dev_config *uart0_config = PINCTRL_DT_DEV_CONFIG_GET(DT_NODELABEL(uart0));
struct pinctrl_dev_config *i2c0_config = PINCTRL_DT_DEV_CONFIG_GET(DT_NODELABEL(i2c0));

static const struct pinctrl_state uart0_alt[] = {
	PINCTRL_DT_STATE_INIT(uart0_alt_default, PINCTRL_STATE_DEFAULT),
#ifdef CONFIG_PM_DEVICE
	PINCTRL_DT_STATE_INIT(uart0_alt_sleep, PINCTRL_STATE_SLEEP),
#endif
};

static const struct pinctrl_state i2c_alt[] = {
	PINCTRL_DT_STATE_INIT(i2c0_alt_default, PINCTRL_STATE_DEFAULT),
#ifdef CONFIG_PM_DEVICE
	PINCTRL_DT_STATE_INIT(i2c0_alt_sleep, PINCTRL_STATE_SLEEP),
#endif
};

static const struct pinctrl_state i2c_default[] = {
	PINCTRL_DT_STATE_INIT(i2c0_default, PINCTRL_STATE_DEFAULT),
#ifdef CONFIG_PM_DEVICE
	PINCTRL_DT_STATE_INIT(i2c0_sleep, PINCTRL_STATE_SLEEP),
#endif
};

/* Sensor related defines =================================================================================*/

// Oversampling rate für den Pressure Sensor 
struct sensor_value oversampling_rate = { 256, 0 }; //{ 8192, 0 }; 256 gewählt für die geringste Genauigkeit 

/* Device Structs =========================================================================================*/

const struct device *pressure_dev		= NULL;
const struct device *imu_dev			= NULL;
const struct device *gpio_dev			= NULL;
const struct device *i2c_dev			= NULL;

/* Global variables =======================================================================================*/
int counter = 0;

/* Pincontrol Functions =================================================================================*/
static int remap_pins_uart()
{
	return pinctrl_update_states(uart0_config, uart0_alt, ARRAY_SIZE(uart0_alt));
}

static int remap_pins_imu()
{
	pm_device_action_run(i2c_dev, PM_DEVICE_ACTION_TURN_OFF); 
	int ret = pinctrl_update_states(i2c0_config, i2c_default, ARRAY_SIZE(i2c_default));
	if (ret < 0) 
	{
			printk("Er ist hier reingelaufen 1\n");
	}
	ret = pinctrl_apply_state_direct(i2c0_config,i2c_default);
	if (ret < 0) 
	{
			printk("Er ist hier reingelaufen 2\n");
	}
	pm_device_action_run(i2c_dev, PM_DEVICE_ACTION_TURN_ON);
	return ret;
}

static int remap_pins_pressure()
{
	pm_device_action_run(pressure_dev, PM_DEVICE_ACTION_TURN_OFF);
	pm_device_action_run(i2c_dev, PM_DEVICE_ACTION_TURN_OFF); 
	int ret = pinctrl_update_states(i2c0_config, i2c_alt, ARRAY_SIZE(i2c_alt));
	if (ret < 0) 
	{
			printk("Er ist hier reingelaufen 1\n");
	}
	ret = pinctrl_apply_state_direct(i2c0_config,i2c_alt);
	if (ret < 0) 
	{
			printk("Er ist hier reingelaufen 2\n");
	}
	pm_device_action_run(pressure_dev, PM_DEVICE_ACTION_TURN_ON);
	pm_device_action_run(i2c_dev, PM_DEVICE_ACTION_TURN_ON);
	return ret;
}

/* Initialization functions =================================================================================*/

/* initialization - pressure sensor */
static void init_pressure(){
	//gpio_pin_set(gpio_dev, VDD_PIN_PRESSURE, 1);
	pressure_dev = DEVICE_DT_GET_ONE(meas_ms5607);

	if (pressure_dev == NULL) {
		printk("Could not find MS5607 device, aborting test.\n");
		return;
	}
	if (!device_is_ready(pressure_dev)) {
		printk("MS5607 device %s is not ready, aborting test.\n",
			pressure_dev->name);
		return;
	}

	if (sensor_attr_set(pressure_dev, SENSOR_CHAN_ALL, SENSOR_ATTR_OVERSAMPLING,
				&oversampling_rate) != 0) {
		printk("Could not set oversampling rate of %d "
			"on MS5837 device, aborting test.",
			oversampling_rate.val1);
		return;
	}
	gpio_pin_set(gpio_dev, VDD_PIN_PRESSURE, 0);
}

/* initialize imu*/
void init_imu(){
	imu_dev = DEVICE_DT_GET_ONE(st_lsm6dso);

	if (!device_is_ready(imu_dev)) {
		printk("%s: device not ready.\n", imu_dev->name);
	}

	if (set_sampling_freq(imu_dev) != 0) {
		printk("sampling frequency cannot be set");
		}
	gpio_pin_set(gpio_dev, VDD_PIN_IMU, 1);
}

/* initialize GPIO*/
static void init_gpio(){
	int ret = 0;
	gpio_dev = device_get_binding(GPIO_DEV_NAME1);

	if (!gpio_dev) {
		printk("Cannot find %s!\n", GPIO_DEV_NAME1);
	}

	//Neccessary for GPIO Sensor Powering
	ret = gpio_pin_configure(gpio_dev, VDD_PIN_IMU, GPIO_OUTPUT_ACTIVE);	
	ret = gpio_pin_configure(gpio_dev, VDD_PIN_PRESSURE, GPIO_OUTPUT_ACTIVE);
	if (ret < 0) {
		printk("Cannot configure %s!\n", GPIO_DEV_NAME1);
	}
	gpio_pin_set(gpio_dev, VDD_PIN_IMU, 1);
	gpio_pin_set(gpio_dev, VDD_PIN_PRESSURE, 1);
}

/* initialization - i2c driver */
static void init_i2c(){
	//gpio_pin_set(gpio_dev, VDD_PIN_PRESSURE, 1);
	i2c_dev = device_get_binding(DEVICE_NAME_I2C);

	if (i2c_dev == NULL) {
		printk("could not find i2c device.\n");
		return;
	}
	if (!device_is_ready(i2c_dev)) {
		printk("i2c interface isnt ready.\n",
			pressure_dev->name);
		return;
	}
}




void main(void)
{
	printk("Hello World!\n");

	init_gpio();
	init_i2c();
	remap_pins_imu();
	init_imu();
	remap_pins_pressure();
	init_pressure();

while (true)
	{
		printk("=========================== Schleifendurchlauf (%i) ===========================\n",counter);
		printk("-------------------------------------------------------------------------------\n");
		counter++;
		remap_pins_pressure();
		gpio_pin_set(gpio_dev, VDD_PIN_PRESSURE, 1);
		k_sleep(K_MSEC(10));
		get_pressure_temperature(pressure_dev,gpio_dev);
		get_pressure(pressure_dev,gpio_dev);
		gpio_pin_set(gpio_dev, VDD_PIN_PRESSURE, 0);
		
		
		gpio_pin_set(gpio_dev, VDD_PIN_IMU, 1);
		k_sleep(K_MSEC(10));
		remap_pins_imu();
		// pm_device_action_run(imu_dev,PM_DEVICE_ACTION_TURN_ON);		
		get_rotation(imu_dev);
		// k_sleep(K_MSEC(10));
		// pm_device_action_run(imu_dev,PM_DEVICE_ACTION_TURN_OFF);
		// gpio_pin_set(gpio_dev, VDD_PIN_IMU, 0);
				
		printk("-------------------------------------------------------------------------------\n");
		k_sleep(K_SECONDS(2));
	}
}

/* Ablage */

// funktion, Initialisierungszeitpunkt und priorität 
// SYS_INIT(remap_pins, PRE_KERNEL_1, CONFIG_REMAP_INIT_PRIORITY);